#!/usr/bin/env python3
# coding: utf-8

from pathlib import Path


class Config:
    PROJECT_PATH = f'{Path(__file__).parent}'
    UBLOCK = f'{PROJECT_PATH}/.addons/ublock_latest.xpi'
    DATA_PATH = f'{PROJECT_PATH}/data'
    LOG_FILE = 'pymirror.log'
    WIN_GECKO = None



#!/usr/bin/env python3
# coding: utf-8

import json
import shlex
import subprocess
import time
from pathlib import Path
from typing import Union, Optional

from dracula import DraculaPalette as Dp

from .config import Config
from .helpers import Shared, console
from .start_driver import StartDrive


class MultiUp:
    def __init__(self, args, file):
        self.args = args
        self.file = file

    def _multiup(self, driver) -> Optional[list]:
        def cURL_request(url: str) -> Union[None, dict]:
            cURL = shlex.split(url)
            out = subprocess.run(cURL, stdout=subprocess.PIPE)
            res = out.stdout.decode('UTF-8').replace('\\', '')
            try:
                res = json.loads(res)
            except json.JSONDecodeError:
                res = None
            return res

        multiup_links = []

        with open(f'{Config.DATA_PATH}/more_links.json') as j:
            more_links = json.load(j)

        server = cURL_request(
            'curl -s https://www.multiup.org/api/get-fastest-server'
        )['server']
        selected_hosts_lst = [
            'filerio.in', 'drop.download', 'download.gg', 'uppit.com',
            'uploadbox.io'
        ]
        limit_n = len(Shared.all_links)
        file_size = Path(self.file).stat().st_size / 1e+6
        for x in selected_hosts_lst:
            if self.args.number:
                if int(self.args.number) <= limit_n:
                    selected_hosts_lst.remove(x)
                    continue
            if file_size > more_links['multiup'][x]['limit']:
                selected_hosts_lst.remove(x)
            else:
                limit_n += 1
        if not selected_hosts_lst:
            return

        if self.args.debug:
            s = ''
        else:
            s = 's'
        selected_hosts = ' '.join(
            [f'-F {x}=true' for x in selected_hosts_lst])
        upload = cURL_request(
            f'curl -{s}F "files[]=@{self.file}" {selected_hosts} {server}')
        if len(upload['files']) == 0:
            upload = cURL_request(f'curl -{s}F "files[]=@{self.file}" '
                                  f'{server}')
        link = upload['files'][0]['url'].replace(
            'download', 'en/mirror')
        driver.get(link)

        i = 0
        j = 0
        es_len = None
        elements = []

        while True:
            time.sleep(1)
            if es_len != len(elements):
                j = 0
            driver.refresh()
            elements = driver.find_elements_by_class_name('host')
            es_len = len(elements)
            if es_len == 2:
                i += 1
            else:
                j += 1
            if j > i + 10:
                break
            elif len(elements) == len(selected_hosts_lst) + 1:
                break

        for e in elements:
            if '(0)' in e.text:
                link = e.get_attribute('link')
                Shared.all_links.append(link)
                multiup_links.append(link)
                console.print(f'[[{Dp.g}] OK [/{Dp.g}]]', link)

        driver.quit()

        return multiup_links

    def upload(self, headless=True):
        if self.args.debug:
            driver = StartDrive(
                cur_module=Path(__file__).name).start_driver(headless)
        else:
            driver = StartDrive(None).start_driver()
        mirroredto_urls = self._multiup(driver)
        return mirroredto_urls



#!/usr/bin/env python3
# coding: utf-8

import concurrent.futures
import time
from pathlib import Path
from typing import Optional

from dracula import DraculaPalette as Dp

from ..helpers import Shared, console, selenium_exceptions
from ..start_driver import StartDrive


class MoreLinks:
    def __init__(self, args, file: str) -> None:
        self.file = file
        self.args = args
        self.driver = StartDrive(None).start_driver()
        self.file_size = Path(self.file).stat().st_size / 1e+6
        self.headless = True

    def usaupload(self) -> Optional[str]:
        if self.file_size > 1000:
            return
        self.driver.get('https://usaupload.com/register_non_user')
        time.sleep(2)
        self.driver.find_element_by_id('add_files_btn').send_keys(self.file)
        self.driver.find_element_by_class_name('upload-button').click()
        while True:
            time.sleep(1)
            try:
                link = self.driver.find_element_by_class_name(
                    'col-xs-4').get_attribute('dtfullurl')
                if link:
                    return link
            except selenium_exceptions:
                continue

    def filesharego(self) -> Optional[str]:
        if self.file_size > 5000:
            return
        self.driver.get('https://www.filesharego.com')
        time.sleep(2)
        for e in self.driver.find_elements_by_class_name('nav-item'):
            if 'Upload File' in e.text:
                e.click()
                break
        time.sleep(2)
        self.driver.find_element_by_class_name('dz-hidden-input').send_keys(
            self.file)
        while True:
            try:
                time.sleep(1)
                link_id = self.driver.find_element_by_id('copy').get_attribute(
                    'data-id')
                link = self.driver.find_element_by_id(link_id).get_attribute(
                    'value')
                if link:
                    break
            except selenium_exceptions:
                link = None
                continue
        return link

    def filepizza(self) -> Optional[str]:
        self.driver.get('https://file.pizza/')
        self.driver.find_element_by_css_selector(
            '.select-file-label > input:nth-child(1)').send_keys(self.file)
        while True:
            time.sleep(1)
            link = self.driver.find_element_by_class_name('short-url').text
            if link:
                break
        link = link.replace('or, for short: ', '')
        return link

    def expirebox(self) -> Optional[str]:
        if self.file_size > 200:
            return
        self.driver.get('https://expirebox.com/')
        time.sleep(2)
        self.driver.find_element_by_id('fileupload').send_keys(self.file)
        while True:
            time.sleep(1)
            try:
                link = self.driver.find_element_by_css_selector(
                    'div.input-group:nth-child(3) > input:nth-child(1)'
                ).get_attribute('value')
                if link:
                    break
            except selenium_exceptions:
                link = None
                continue
        return link

    def filepost(self) -> Optional[str]:
        if self.file_size > 3000:
            return
        self.driver.get('https://filepost.io/')
        self.driver.find_element_by_css_selector(
            '.drop-region > input:nth-child(4)').send_keys(self.file)
        while True:
            time.sleep(1)
            try:
                e = self.driver.find_element_by_css_selector(
                    'div.buttons:nth-child(3) > a:nth-child(1)')
                link = e.get_attribute('href').split('&body=')[1]
                if link:
                    break
            except selenium_exceptions:
                link = None
                continue
        return link

    def upload(self):
        args_ = self.args, self.file

        hosts = [
            MoreLinks(*args_).filepizza,
            MoreLinks(*args_).usaupload,
            MoreLinks(*args_).filesharego,
            MoreLinks(*args_).expirebox,
            MoreLinks(*args_).filepost
        ]

        if self.args.number:
            if int(self.args.number) < len(Shared.all_links) + 5:
                left = int(self.args.number) - len(Shared.all_links)
                hosts = hosts[:left]

        with concurrent.futures.ThreadPoolExecutor() as executor:
            urls = []
            results = [executor.submit(fun) for fun in hosts]
            for future in concurrent.futures.as_completed(results):
                link = future.result()
                urls.append(link)
                Shared.all_links.append(link)
                console.print(f'[[{Dp.g}] OK [/{Dp.g}]]', link)
        return urls



#!/usr/bin/env python3
# coding: utf-8

import json
import multiprocessing as mp
import os
import platform
import re
import shlex
import shutil
import signal
import socket
import statistics
import subprocess
import sys
import tarfile
import time
import traceback
from pathlib import Path
from typing import Union, NoReturn, Optional

from dracula import DraculaPalette as Dp
from rich.panel import Panel

from .config import Config
from .handlers import KeyboardInterruptHandler
from .helpers import Shared, console, logger
from .mirroredto import Mirroredto
from .multiup import MultiUp
from .experimental.more_links import MoreLinks


class PyMirror:
    def __init__(self) -> None:
        pass

    @staticmethod
    def custom_error_traceback(exception: Exception,
                               error_msg: str,
                               log: bool = False) -> list:
        if log:
            logger.error(error_msg)
        tb = traceback.format_exception(None, exception,
                                        exception.__traceback__)
        if log:
            logger.error(f'{"-" * 10} Start of error traceback {"-" * 10}')
            for ln in tb:
                logger.error(ln.replace('\n', ''))
            logger.error(f'{"-" * 10} End of error traceback {"-" * 10}')
        return tb

    @staticmethod
    def tgz(input_folder: str) -> str:
        out_file = f'{Path(input_folder).parent}/' \
                   f'{Path(input_folder).name}.tar.gz '
        with tarfile.open(out_file, 'w:gz') as t:
            t.add(input_folder, arcname=Path(input_folder).name)
        return out_file

    @staticmethod
    def initializer() -> NoReturn:
        signal.signal(signal.SIGINT, signal.SIG_IGN)

    @staticmethod
    def return_ips(data: dict) -> list:
        ips = []
        for k, v in data.items():
            try:
                server = data[k]['server'].split('/')[2]
            except IndexError:
                server = data[k]['server']
            ip = socket.gethostbyname(server)
            ips.append((ip, k))
        return ips

    @staticmethod
    def ping(ip: str) -> bool:
        if platform.system() == 'Windows':
            response = os.system(f'ping -n 1 {ip[0]} > /dev/null 2>&1')
        else:
            response = os.system(f'ping -c 1 {ip[0]} > /dev/null 2>&1')
        if response in [0, 256, 512]:
            console.print(
                f'[[{Dp.g}] OK [/{Dp.g}]] [{Dp.c}]{ip[1]}[/{Dp.c}] is online!')
            logger.info(f'{ip[1]} is online!')
            return True
        else:
            console.print(
                f'[[{Dp.r}] ERROR! [/{Dp.r}]] [{Dp.c}]{ip[1]}[/{Dp.c}] is '
                'down!'
            )
            logger.warning(f'{ip[1]} is offline!')
            return False

    @staticmethod
    def curl(data: dict, server: str, file: str) -> Optional[str]:
        file_size = Path(file).stat().st_size / 1e+6
        size_limit = data[server]['limit']
        if file_size > size_limit:
            return
        srv = data[server]['server']
        keys = data[server]['keys']
        flags = data[server]['flags']
        parameter = data[server]['parameter']
        cURL = f'curl {flags} "{parameter}{file}" {srv}'
        cURL = shlex.split(cURL)
        out = subprocess.run(cURL, stdout=subprocess.PIPE)
        try:
            out = json.loads(out.stdout)
        except json.decoder.JSONDecodeError:
            out = out.stdout.decode('UTF-8').strip('\n')

        keyslist = {
            0: 'out',
            1: 'out[keys[0]]',
            2: 'out[keys[0]][keys[1]]',
            3: 'out[keys[0]][keys[1]][keys[2]]'
        }

        link = eval(keyslist[len(keys)])

        if server == 'oshi':
            link = out.split('\n')[1].replace('DL: ', '')

        return link

    @staticmethod
    def api_uploads(args, data: dict, responses: list, file: str) -> list:
        times = []
        api_uploads_links = []
        for n, ((k, _), res) in enumerate(zip(data.items(), responses)):
            if args.number:
                if n == int(args.number):
                    break
            if args.debug:
                if n == 1:
                    break
            if res is False:
                continue
            try:
                signal.signal(signal.SIGALRM, lambda x, y: 1 / 0)
                start = time.time()
                if len(times) > 2:
                    signal.alarm(int(statistics.mean(times)) + 5)
                link = PyMirror.curl(data, k, file)
                if 'bad gateway' in link.lower() or 'error' in link.lower():
                    raise Exception
                console.print(f'[[{Dp.g}] OK [/{Dp.g}]]', link)
                Shared.all_links.append(link)
                api_uploads_links.append(link)
                logger.info(f'[ OK ] {link}')
                times.append(time.time() - start)
            except ZeroDivisionError:
                if args.log:
                    logger.error(f'{k} Timed out!')
            except Exception as e:
                PyMirror.custom_error_traceback(e,
                                                f'[ ERROR! ] Error in {k}...',
                                                log=args.log)
            finally:
                signal.alarm(0)
        return api_uploads_links

    @staticmethod
    def _match_links(links_raw: list) -> list:
        regex = re.compile(
            r'^(?:http|ftp)s?://'
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+'
            r'(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'
            r'localhost|'
            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'
            r'(?::\d+)?'
            r'(?:/?|[/?]\S+)$', re.IGNORECASE)

        links = []
        for link in links_raw:
            if re.match(regex, link) is not None:
                links.append(link)
        return links

    @staticmethod
    def style_output(args, links_dict: dict) -> Union[list, str]:
        names = list(links_dict.keys())
        links = list(links_dict.values())
        style = args.style
        if style == 'list':
            output = '\n'.join(links)
        elif style == 'markdown':
            output = '\n'.join(
                [f'- [{name}]({link})' for name, link in zip(names, links)])
        elif style == 'reddit':
            output = ' | '.join(
                [f'[Mirror {n + 1}]({link})' for n, link in enumerate(links)])
        else:
            output = '\n'.join(links)
        return output

    @staticmethod
    def uploader(args) -> Union[list, str]:
        start_time = time.time()
        signal.signal(signal.SIGINT,
                      KeyboardInterruptHandler.keyboardInterruptHandler)

        if args.experimental and not args.more_links:
            raise Exception('You need to add the `--more-links` flag to use `--experimental`')

        with open(f'{Config.DATA_PATH}/servers_data.json') as j:
            data = json.load(j)

        if args.log:
            logger.remove()
            logger.add(Config.LOG_FILE, level='DEBUG')
            logger.add(sys.stderr, level='ERROR')

        console.print('Press `CTRL+C` at any time to quit.', style='#f1fa8c')

        if args.check_status:
            console.rule('Checking servers status...')
            ips = PyMirror.return_ips(data)
            cpus = mp.cpu_count() - 1
            with mp.Pool(cpus, initializer=PyMirror.initializer) as p:
                responses = p.map(PyMirror.ping, ips)
        else:
            responses = []

        console.rule('Uploading...')

        file = args.input
        if not Path(file).exists():
            raise FileNotFoundError('If you are sure the path is correct, '
                                    'rename the file removing any illegal '
                                    'characters, then try again.')

        if Path(file).is_dir():
            file = PyMirror.tgz(file)

        if len(responses) == 0:
            responses = [True] * len(data.keys())

        PyMirror.api_uploads(args, data, responses, file)

        if args.more_links:
            file_resolved = str(Path(file).resolve())
            Mirroredto(args, file_resolved).upload()
            MultiUp(args, file_resolved).upload()
            if args.experimental:
                MoreLinks(args, file_resolved).upload()

        links_dict = {}
        for link in Shared.all_links:
            domain = link.split('/')[:-1][2].replace('www.', '')
            name = '.'.join(domain.split('.')[0:])
            if len(name) == 1:
                name = '.'.join(domain.split('.')[1:])
            elif len(name.split('.')) == 3:
                name = '.'.join(domain.split('.')[1:])
            links_dict.update({name: link})

        output = PyMirror.style_output(args, links_dict)

        if args.delete:
            if Path(args.input).is_dir():
                shutil.rmtree(args.input)
            else:
                os.remove(file)
        if Path(file).suffixes == ['.tar', '.gz']:
            os.remove(file)
        console.rule(f'Results: {len(Shared.all_links)}')
        print(output)
        os.rename(file, file)
        for x in Shared.all_links:
            logger.info(x)
        run_time = time.strftime('%H:%M:%S',
                                 time.gmtime(time.time() - start_time))
        h, m, s = [int(_) for _ in run_time.split(':')]
        console.print(
            Panel.fit(f'[{Dp.k}]Process took[{Dp.k}] [{Dp.y}]{h}h {m}m {s}s'))
        console.rule('END')
        return output



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pymirror', 'pymirror.experimental']

package_data = \
{'': ['*'], 'pymirror': ['.addons/*', 'data/*']}

entry_points = \
{'console_scripts': ['pymirror = pymirror.run:main']}

setup(name='pymirror',
      version='0.3.3',
      description='pymirror',
      author=None,
      author_email='Alyetama <56323389+Alyetama@users.noreply.github.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      entry_points=entry_points,
     )



#!/usr/bin/env python3
# coding: utf-8

import sys
from pathlib import Path

from .cli import cli
from .main import PyMirror


def main():
    args = cli()
    PyMirror().uploader(args)


if __name__ == '__main__':
    testing = True
    if testing:
        foo = Path('foo.txt')
        if not foo.exists():
            with open(str(foo), 'w') as f:
                f.write('foo\n')
        sys.argv[1:] = ['-i', str(foo), '-m', '--debug']
        main()
        foo.unlink()
    else:
        main()



#!/usr/bin/env python3
# coding: utf-8
import inspect

import loguru
from rich.console import Console
import selenium.common.exceptions


class Shared:
    all_links = []
    pids = []


def logger():
    logger_ = loguru.logger
    logger_.remove()
    return logger_


def selenium_exceptions_classes():
    selenium_exceptions_list = inspect.getmembers(selenium.common.exceptions,
                                                  predicate=inspect.isclass)
    return list(zip(*selenium_exceptions_list))[1]


console = Console()
logger = logger()
selenium_exceptions = selenium_exceptions_classes()
